from typing import Optional

from cmai.config.settings import settings
from cmai.utils.git_staged_analyzer import GitStagedAnalyzer
from cmai.providers.base import AIResponse
from cmai.providers.provider_factory import create_provider


class Normalizer:
    def __init__(self) -> None:
        pass

    async def normalize_commit(
        self,
        user_input: str,
        prompt_template: str,
        language: Optional[str] = None,
        repo_path: Optional[str] = None,
    ) -> AIResponse:
        git_analyzer = GitStagedAnalyzer(repo_path=repo_path)
        cached_diff = git_analyzer.get_cached_diff()

        if not cached_diff or len(cached_diff) == 0:
            raise ValueError("No staged changes found in the repository.")

        diff_content = "\n".join(cached_diff)
        prompt = (
            prompt_template.replace("{user_input}", user_input)
            .replace("{diff_content}", diff_content)
            .replace("{language}", language or settings.RESPONSE_LANGUAGE)
        )

        provider = create_provider()

        response = await provider.normalize_commit(prompt, diff_content=diff_content)

        return response



import subprocess
from typing import List, Optional
from pathlib import Path

from cmai.core.get_logger import LoggerFactory
from cmai.config.settings import settings


class GitStagedAnalyzer:
    MAX_DIFF_SIZE = settings.MAX_DIFF_LENGTH
    IGNORED_EXTENSIONS = {
        ".lock",
        ".png",
        ".jpg",
        ".jpeg",
        ".gif",
        ".ico",
        ".pdf",
        ".zip",
        ".tar",
        ".gz",
        ".pyc",
        ".bin",
        ".exe",
        ".dll",
        ".so",
    }

    def __init__(self, repo_path: Optional[str] = None) -> None:
        self.logger = LoggerFactory().get_logger("GitAddLogger")
        if repo_path is None:
            repo_path = str(Path.cwd().resolve())
        self.logger.info(f"Initializing GitStagedAnalyzer for repo: {repo_path}")
        self.repo_path = Path(repo_path).resolve()

    def get_cached_diff(self) -> Optional[List[str]]:
        try:
            modified_files = self._get_modified_files()
            if not modified_files:
                self.logger.info("No staged changes found.")
                return None
            self.logger.debug(f"Staged files: {modified_files}")

            detailed_diffs = []
            current_length = 0

            for file_name in modified_files:
                detailed_diff = self._get_detailed_diff(file_name)
                if not detailed_diff:
                    continue

                # Skip binary files if git reports them as such
                if "Binary files" in detailed_diff and "differ" in detailed_diff:
                    continue

                entry = f"{file_name}:\n{detailed_diff}"
                entry_len = len(entry)

                if current_length + entry_len > self.MAX_DIFF_SIZE:
                    self.logger.warning(
                        "Staged changes too large, returning file list only."
                    )
                    summary = [
                        f"Total staged changes exceed {self.MAX_DIFF_SIZE} characters. Modified files list:"
                    ]
                    summary.extend([f"- {f}" for f in modified_files])
                    return summary

                detailed_diffs.append(entry)
                current_length += entry_len

            return detailed_diffs
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Error checking git status: {e}")
            return None

    def _get_modified_files(self) -> List[str]:
        """获取已暂存的修改文件列表"""
        try:
            stat_result = subprocess.run(
                ["git", "diff", "--cached", "--name-only"],
                cwd=self.repo_path,
                capture_output=True,
                text=True,
                check=True,
                encoding="utf-8",
                errors="replace",
            )
            files = stat_result.stdout.strip().splitlines()
            return [
                f
                for f in files
                if not any(f.endswith(ext) for ext in self.IGNORED_EXTENSIONS)
            ]
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Error getting modified files: {e}")
            return []

    def _get_detailed_diff(self, file_name: str) -> str:
        """获取指定文件的详细差异"""
        try:
            diff_result = subprocess.run(
                ["git", "diff", "--cached", file_name],
                cwd=self.repo_path,
                capture_output=True,
                text=True,
                check=True,
                encoding="utf-8",
                errors="replace",
            )
            self.logger.debug(
                f"Detailed diff for {file_name}: {diff_result.stdout.strip()}"
            )
            return diff_result.stdout.strip()
        except subprocess.CalledProcessError as e:
            self.logger.warning(f"Error getting detailed diff for {file_name}: {e}")
            self.logger.debug("Try find the change type of the file")
            status_result = subprocess.run(
                ["git", "status", "--porcelain", file_name],
                cwd=self.repo_path,
                capture_output=True,
                text=True,
                check=True,
                encoding="utf-8",
                errors="replace",
            )
            match status_result.stdout.strip()[0]:
                case "M":
                    self.logger.debug(f"File {file_name} is modified.")
                    return f"{file_name} has been modified."
                case "A":
                    self.logger.debug(f"File {file_name} is added.")
                    return f"{file_name} has been added."
                case "D":
                    self.logger.debug(f"File {file_name} is deleted.")
                    return f"{file_name} has been deleted."
                case _:
                    self.logger.debug(f"File {file_name} has unknown status.")
                    return f"{file_name} has an unknown status."



import os
from zai import ZhipuAiClient
from zai.types.chat import ChatCompletionChunk

from cmai.providers.base import AIResponse

from .base import BaseAIClient
from cmai.config.settings import settings
from cmai.core.get_logger import LoggerFactory


class ZhipuAiProvider(BaseAIClient):
    """智谱 AI 实现"""

    def __init__(
        self, api_key: str | None = None, model: str | None = None, **kwargs
    ) -> None:
        super().__init__(api_key=api_key, model=model, **kwargs)
        self.logger = LoggerFactory().get_logger("ZhipuAiProvider")
        self.stream_logger = LoggerFactory().get_stream_logger("ZhipuAiProvider")

        self.api_key = api_key or settings.API_KEY or os.getenv("CMAI_API_KEY")

        if not self.api_key:
            raise ValueError("API key is required for ZhipuAiProvider.")

        base_url = settings.API_BASE or os.getenv("ZHIPUAI_BASE_URL") or None

        self.client = ZhipuAiClient(api_key=self.api_key, base_url=base_url, **kwargs)

        self.model = model or settings.MODEL or "glm-4.5-flash"

    def validate_config(self) -> bool:
        return bool(self.api_key)

    async def normalize_commit(self, prompt: str, **kargs) -> AIResponse:
        diff_content = kargs.pop("diff_content", None)
        if diff_content:
            log_prompt = prompt.replace(
                diff_content, f"[Diff content hidden, length: {len(diff_content)}]"
            )
        else:
            log_prompt = prompt
        self.logger.debug(f"Normalizing commit with prompt: {log_prompt}")

        completion = self.client.chat.completions.create(
            model=self.model or "qwen-turbo-latest",
            messages=[{"role": "user", "content": prompt}],
            stream=True,
            thinking={"type": "enabled" if settings.ENABLE_THINKING else "disabled"},
            **kargs,
        )

        reason = ""
        response = ""
        is_answering = False
        is_reasoning = False
        usage = None
        for chunk in completion:
            if isinstance(chunk, ChatCompletionChunk):
                if chunk.choices[0]:
                    delta = chunk.choices[0].delta
                    if (
                        hasattr(delta, "reasoning_content")
                        and getattr(delta, "reasoning_content", None) is not None
                    ):
                        if not is_reasoning:
                            self.logger.info(
                                "Detected reasoning content...\nPlease wait..."
                            )
                            is_reasoning = True
                        self.stream_logger.info(getattr(delta, "reasoning_content", ""))
                        reason += getattr(delta, "reasoning_content", "")
                    else:
                        if not is_answering:
                            self.stream_logger.info("\n")
                            self.logger.debug("Starting to answer...")
                            is_answering = True
                        self.stream_logger.info(chunk.choices[0].delta.content or "")
                        response += chunk.choices[0].delta.content or ""
                elif chunk.usage:
                    usage = chunk.usage.total_tokens
                    self.logger.debug(f"Received usage info: {usage} tokens")
                else:
                    self.logger.warning(f"Unexpected chunk received: {chunk}")
            else:
                self.logger.warning(f"Unexpected chunk received: {chunk}")

        if usage is None:
            self.logger.warning("No usage information received")
            usage = 0

        self.logger.info(f"Final normalized commit message: {response.strip()}")

        return AIResponse(
            content=response.strip(),
            model=self.model or "glm-4.5-flash",
            provider="zai",
            tokens_used=usage,
        )



from typing import Optional
import os

from anthropic import Anthropic

from cmai.config.settings import settings
from cmai.providers.base import BaseAIClient, AIResponse
from cmai.core.get_logger import LoggerFactory


class AnthropicProvider(BaseAIClient):
    """Anthropic Claude 客户端实现"""

    def __init__(
        self,
        api_key: Optional[str] = None,
        model: Optional[str] = None,
        **kwargs,
    ) -> None:
        """
        初始化 Anthropic 客户端

        Args:
            api_key (str): API密钥，用于认证请求，默认从环境变量中获取。
            model (str, optional): 使用的模型名称，默认为"claude-haiku-4-5-20251001"。
            **kwargs: 其他可选参数，具体取决于不同的API配置。
        """
        super().__init__(api_key=api_key, model=model, **kwargs)
        self.logger = LoggerFactory().get_logger("AnthropicProvider")
        self.stream_logger = LoggerFactory().get_stream_logger("AnthropicProvider")

        self.api_key = api_key or settings.API_KEY or os.getenv("ANTHROPIC_API_KEY")

        # 验证API Key是否存在
        if not self.api_key:
            raise ValueError("API key is required for AnthropicProvider.")

        # 获取 API Base
        base_url = settings.API_BASE if settings.API_BASE else None

        # 获取 model
        if not model:
            model = settings.MODEL or "claude-3-5-sonnet-20241022"

        self.model = model

        # 创建 Anthropic 客户端，如果有自定义 base_url 则使用
        if base_url:
            self.client = Anthropic(api_key=self.api_key, base_url=base_url, **kwargs)
        else:
            self.client = Anthropic(api_key=self.api_key, **kwargs)

        self.provider = "anthropic" if not settings.PROVIDER else settings.PROVIDER

    def validate_config(self) -> bool:
        return bool(self.api_key)

    async def normalize_commit(self, prompt: str, **kargs) -> AIResponse:
        diff_content = kargs.pop("diff_content", None)
        if diff_content:
            log_prompt = prompt.replace(
                diff_content, f"[Diff content hidden, length: {len(diff_content)}]"
            )
        else:
            log_prompt = prompt
        self.logger.debug(f"Normalizing commit with prompt: {log_prompt}")

        # 构建请求参数
        create_params = {
            "model": self.model or "claude-haiku-4-5-20251001",
            "messages": [{"role": "user", "content": prompt}],
            "max_tokens": settings.MAX_TOKEN,
            "stream": True,
        }

        # 只在启用 thinking 时才添加该参数
        if settings.ENABLE_THINKING:
            create_params["thinking"] = {
                "type": "enabled",
                "budget_tokens": settings.THINKING_BUDGET,
            }

        stream = self.client.messages.create(**create_params)

        response = ""
        is_answering = False
        input_tokens = 0
        output_tokens = 0

        # 处理流式响应
        with stream as completion:
            for event in completion:
                # 处理消息开始事件
                if event.type == "message_start":
                    if hasattr(event, "message") and hasattr(event.message, "usage"):
                        input_tokens = event.message.usage.input_tokens
                        self.logger.debug(f"Input tokens: {input_tokens}")

                # 处理内容块增量
                elif event.type == "content_block_delta":
                    if hasattr(event, "delta") and hasattr(event.delta, "text"):
                        if not is_answering:
                            self.logger.debug("Starting to answer...")
                            is_answering = True
                        text = event.delta.text
                        self.stream_logger.info(text)
                        response += text

                # 处理消息结束事件
                elif event.type == "message_delta":
                    if hasattr(event, "usage"):
                        output_tokens = event.usage.output_tokens
                        self.logger.debug(f"Output tokens: {output_tokens}")

        total_tokens = input_tokens + output_tokens
        if total_tokens == 0:
            self.logger.warning("No usage information received")

        self.logger.info(f"Final normalized commit message: {response.strip()}")

        return AIResponse(
            content=response.strip(),
            model=self.model or "claude-haiku-4-5-20251001",
            provider=self.provider,
            tokens_used=total_tokens if total_tokens > 0 else None,
        )
